package sun.security.provider.certpath;

import java.security.cert.X509CRLSelector;
import java.security.cert.CRLSelector;
import java.security.cert.CertStoreException;
import java.security.cert.X509CertSelector;
import java.security.cert.CertSelector;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.util.Collections;
import java.util.HashSet;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.HashMap;
import java.util.Collection;
import java.security.InvalidAlgorithmParameterException;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.CertStoreParameters;
import java.security.cert.CRL;
import java.security.cert.Certificate;
import java.util.Set;
import javax.security.auth.x500.X500Principal;
import java.util.Map;
import java.security.cert.CertStoreSpi;

public class IndexedCollectionCertStore extends CertStoreSpi
{
    private Map<X500Principal, Object> certSubjects;
    private Map<X500Principal, Object> crlIssuers;
    private Set<Certificate> otherCertificates;
    private Set<CRL> otherCRLs;
    
    public IndexedCollectionCertStore(CertStoreParameters params) throws InvalidAlgorithmParameterException {
        super(params);
        if (!(params instanceof CollectionCertStoreParameters)) {
            throw new InvalidAlgorithmParameterException("parameters must be CollectionCertStoreParameters");
        }
        Collection<Object> collection = ((CollectionCertStoreParameters)params).getCollection();
        if (collection == null) {
            throw new InvalidAlgorithmParameterException("Collection must not be null");
        }
        this.buildIndex(collection);
        return;
    }
    
    private void buildIndex(Collection<?> collection) {
        this.certSubjects = new HashMap();
        this.crlIssuers = new HashMap();
        this.otherCertificates = null;
        this.otherCRLs = null;
        for (Object next : collection) {
            if ((next instanceof X509Certificate)) {
                this.indexCertificate((X509Certificate)next);
            }
            else if ((next instanceof X509CRL)) {
                this.indexCRL((X509CRL)next);
            }
            else if ((next instanceof Certificate)) {
                if (this.otherCertificates == null) {
                    this.otherCertificates = new HashSet();
                }
                this.otherCertificates.add((Certificate)next);
            }
            else if ((next instanceof CRL)) {
                if (this.otherCRLs == null) {
                    this.otherCRLs = new HashSet();
                }
                this.otherCRLs.add((CRL)next);
            }
            else {
                continue;
            }
        }
        if (this.otherCertificates == null) {
            this.otherCertificates = Collections.emptySet();
        }
        if (this.otherCRLs == null) {
            this.otherCRLs = Collections.emptySet();
        }
    }
    
    private void indexCertificate(X509Certificate x509Certificate) {
        X500Principal subjectX500Principal = x509Certificate.getSubjectX500Principal();
        Object put = this.certSubjects.put(subjectX500Principal, x509Certificate);
        if (put != null) {
            if ((put instanceof X509Certificate)) {
                if (x509Certificate.equals(put)) {
                    return;
                }
                ArrayList list2 = new ArrayList(2);
                list2.add(x509Certificate);
                list2.add((X509Certificate)put);
                this.certSubjects.put(subjectX500Principal, list2);
            }
            else {
                List<X509Certificate> list = (List)put;
                if (!list.contains(x509Certificate)) {
                    list.add(x509Certificate);
                }
                this.certSubjects.put(subjectX500Principal, list);
            }
        }
    }
    
    private void indexCRL(X509CRL x509CRL) {
        X500Principal issuerX500Principal = x509CRL.getIssuerX500Principal();
        Object put = this.crlIssuers.put(issuerX500Principal, x509CRL);
        if (put != null) {
            if ((put instanceof X509CRL)) {
                if (x509CRL.equals(put)) {
                    return;
                }
                ArrayList list2 = new ArrayList(2);
                list2.add(x509CRL);
                list2.add((X509CRL)put);
                this.crlIssuers.put(issuerX500Principal, list2);
            }
            else {
                List<X509CRL> list = (List)put;
                if (!list.contains(x509CRL)) {
                    list.add(x509CRL);
                }
                this.crlIssuers.put(issuerX500Principal, list);
            }
        }
    }
    
    @Override
    public Collection<? extends Certificate> engineGetCertificates(CertSelector certSelector) throws CertStoreException {
        if (certSelector == null) {
            HashSet<Certificate> set2 = new HashSet();
            this.matchX509Certs(new X509CertSelector(), set2);
            set2.addAll(this.otherCertificates);
            return set2;
        }
        if (!(certSelector instanceof X509CertSelector)) {
            HashSet<Certificate> set = new HashSet();
            this.matchX509Certs(certSelector, set);
            for (Certificate certificate2 : this.otherCertificates) {
                if (certSelector.match(certificate2)) {
                    set.add(certificate2);
                }
                else {
                    continue;
                }
            }
            return set;
        }
        if (this.certSubjects.isEmpty()) {
            return Collections.emptySet();
        }
        X509CertSelector x509CertSelector = (X509CertSelector)certSelector;
        X509Certificate certificate = x509CertSelector.getCertificate();
        X500Principal x500Principal;
        if (certificate != null) {
            x500Principal = certificate.getSubjectX500Principal();
        }
        else {
            x500Principal = x509CertSelector.getSubject();
        }
        if (x500Principal != null) {
            Object value = this.certSubjects.get(x500Principal);
            if (value == null) {
                return Collections.emptySet();
            }
            if ((value instanceof X509Certificate)) {
                X509Certificate x509Certificate = (X509Certificate)value;
                if (x509CertSelector.match(x509Certificate)) {
                    return Collections.singleton(x509Certificate);
                }
                return Collections.emptySet();
            }
            else {
                List<Object> list = (List)value;
                HashSet o2 = new HashSet(16);
                for (X509Certificate cert : list) {
                    if (x509CertSelector.match(cert)) {
                        o2.add(cert);
                    }
                    else {
                        continue;
                    }
                }
                return o2;
            }
        }
        else {
            HashSet o = new HashSet(16);
            this.matchX509Certs(x509CertSelector, o);
            return o;
        }
    }
    
    private void matchX509Certs(CertSelector certSelector, Collection<Certificate> collection) {
        for (Object next : this.certSubjects.values()) {
            if ((next instanceof X509Certificate)) {
                X509Certificate x509Certificate2 = (X509Certificate)next;
                if (certSelector.match(x509Certificate2)) {
                    collection.add(x509Certificate2);
                }
                else {
                    continue;
                }
            }
            else {
                for (X509Certificate x509Certificate : (List)next) {
                    if (certSelector.match(x509Certificate)) {
                        collection.add(x509Certificate);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    @Override
    public Collection<CRL> engineGetCRLs(CRLSelector crlSelector) throws CertStoreException {
        if (crlSelector == null) {
            HashSet<CRL> set4 = new HashSet();
            this.matchX509CRLs(new X509CRLSelector(), set4);
            set4.addAll(this.otherCRLs);
            return set4;
        }
        if (!(crlSelector instanceof X509CRLSelector)) {
            HashSet<CRL> set3 = new HashSet();
            this.matchX509CRLs(crlSelector, set3);
            for (CRL crl : this.otherCRLs) {
                if (crlSelector.match(crl)) {
                    set3.add(crl);
                }
                else {
                    continue;
                }
            }
            return set3;
        }
        if (this.crlIssuers.isEmpty()) {
            return Collections.emptySet();
        }
        X509CRLSelector x509CRLSelector = (X509CRLSelector)crlSelector;
        Collection<X500Principal> issuers = x509CRLSelector.getIssuers();
        if (issuers != null) {
            HashSet set2 = new HashSet(16);
            Iterator<Object> iterator = issuers.iterator();
            while (iterator.hasNext()) {
                Object value = this.crlIssuers.get((Object)iterator.next());
                if (value == null) {
                    continue;
                }
                if ((value instanceof X509CRL)) {
                    X509CRL x509CRL2 = (X509CRL)value;
                    if (x509CRLSelector.match(x509CRL2)) {
                        set2.add(x509CRL2);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    for (X509CRL x509CRL : (List)value) {
                        if (x509CRLSelector.match(x509CRL)) {
                            set2.add(x509CRL);
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            return set2;
        }
        HashSet set = new HashSet(16);
        this.matchX509CRLs(x509CRLSelector, set);
        return set;
    }
    
    private void matchX509CRLs(CRLSelector crlSelector, Collection<CRL> collection) {
        for (Object next : this.crlIssuers.values()) {
            if ((next instanceof X509CRL)) {
                X509CRL x509CRL2 = (X509CRL)next;
                if (crlSelector.match(x509CRL2)) {
                    collection.add(x509CRL2);
                }
                else {
                    continue;
                }
            }
            else {
                for (X509CRL x509CRL : (List)next) {
                    if (crlSelector.match(x509CRL)) {
                        collection.add(x509CRL);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
}
